#!/bin/bash
#
# /root/scripts/dalybms/dalybms.sh
#
# Tested on : Daly BMS 60/30AH Lifepo4 8S 24v 60A R25T1-GK24 Common port BT/UART using a UART to USB cable.
# Date      : 18-03-2022
# Version   : 00,00.00,000.1
#
# Created by ILoveIOT 2022"
#
####################################################################################################################################

# Working directory, place where this script Daly BMS are
WORKINGDIR="/root/scripts/dalybms"
# Daly-bms-cli directory, you can check with "whereis daly-bms-cli"
DALYBMSDIR="/usr/local/bin"

# USB/Serial port 
USBPORT="/dev/ttyUSB0"

# Domoticz IP address
DOMOTICZIP="127.0.0.1"
DOMOTICZPORT="8080"

# Domoticz IDX (make a Domoticz dummy device first)
# Domoticz dummy Voltage (Lithium Voltage)
TOTALVOLTAGEIDX="354"
# Domoticz dummy 1 fase AMP (Lithium Current)
TOTALCURRENTIDX="355"
# Domoticz dummy value % (Lithium SOC)
SOCIDX="356"
# Domoticz dummy 1 fase AMP (Lithium Ah Left)
CAPACITYAHLEFTIDX="357"
# Domoticz dummy temp (Lithium Temp)
TEMPERATUREIDX="358"
# Domoticz dummy Custom Sensor (Axis X) (Lithium Cycles)
CYCLESIDX="359"
# Domoticz dummy Voltage (Lithium Cell 1-8)
CELLVOLTAGE1IDX="360"
CELLVOLTAGE2IDX="361"
CELLVOLTAGE3IDX="362"
CELLVOLTAGE4IDX="363"
CELLVOLTAGE5IDX="364"
CELLVOLTAGE6IDX="365"
CELLVOLTAGE7IDX="366"
CELLVOLTAGE8IDX="367"
# Domoticz dummy Voltage (Lithium Cell Deviation)
CELLDEVIATIONIDX="368"
# Domoticz dummy switch for the state of the charging mosfet (Daly BMS state charging)
CHARGINGMOSFETIDX="369"
# Domoticz dummy switch for the state of the discharging mosfet (Daly BMS state discharging)
DISCHARGINGMOSFETIDX="370"

####################################################################################################################################
####################################################################################################################################

INFO="This script will execute the 'daly-bms-cli -d $USBPORT --all' using UART and put the output to a file, 
get the value's from the output file, show them on console if needed, and send them to Domoticz, this is super simple, easy to change.

Many thanks to Dreadnought https://github.com/dreadnought/python-daly-bms
Requirements : apt install python3-pip curl - pip3 install dalybms - pip3 install pyserial

For console use          : $WORKINGDIR/dalybms.sh
For crontab use          : $WORKINGDIR/dalybms.sh >> /dev/null 2>&1
View content output file : cat $WORKINGDIR/dalybms.output.txt
For all the info         : daly-bms-cli -d /dev/ttyUSB0 --all
For the help info        : daly-bms-cli --help

1. Change the serial port (/dev/ttyUSB0)
2. Change the Domoticz IP & Port & IDX
3. Optional : Change the working directory, where you wanna place the files"

####################################################################################################################################

# The txt output file from reading from the Daly BMS
OUTPUTFILE="$WORKINGDIR/dalybms.output.txt"

# Generate the txt file with controller output
$DALYBMSDIR/daly-bms-cli -d $USBPORT --all > $OUTPUTFILE

# Getting the value's from the output file, and make them readable and put them into a variable.
TOTALVOLTAGE=`cat $OUTPUTFILE | sed -n 3p | awk '{print $2}' | sed 's/.$//'`
TOTALCURRENT=`cat $OUTPUTFILE | sed -n 4p | awk '{print $2}' | sed 's/.$//'`
SOC=`cat $OUTPUTFILE | sed -n 5p | awk '{print $2}'`
HIGHESTVOLTAGE=`cat $OUTPUTFILE | sed -n 8p | awk '{print $2}' | sed 's/.$//'`
HIGHESTCELL=`cat $OUTPUTFILE | sed -n 9p | awk '{print $2}' | sed 's/.$//'`
LOWESTVOLTAGE=`cat $OUTPUTFILE | sed -n 10p | awk '{print $2}' | sed 's/.$//'`
LOWESTCELL=`cat $OUTPUTFILE | sed -n 11p | awk '{print $2}'`
TEMPERATURE=`cat $OUTPUTFILE | sed -n 46p | awk '{print $2}'`
CHARGINGMOSFET=`cat $OUTPUTFILE | sed -n 21p | awk '{print $2}' | sed 's/.$//'`
DISCHARGINGMOSFET=`cat $OUTPUTFILE | sed -n 22p | awk '{print $2}' | sed 's/.$//'`
CAPACITYAHLEFT=`cat $OUTPUTFILE | sed -n 23p | awk '{print $2}'`
CYCLES=`cat $OUTPUTFILE | sed -n 33p | awk '{print $2}'`
CELLVOLTAGE1=`cat $OUTPUTFILE | sed -n 36p | awk '{print $2}' | sed 's/.$//'`
CELLVOLTAGE2=`cat $OUTPUTFILE | sed -n 37p | awk '{print $2}' | sed 's/.$//'`
CELLVOLTAGE3=`cat $OUTPUTFILE | sed -n 38p | awk '{print $2}' | sed 's/.$//'`
CELLVOLTAGE4=`cat $OUTPUTFILE | sed -n 39p | awk '{print $2}' | sed 's/.$//'`
CELLVOLTAGE5=`cat $OUTPUTFILE | sed -n 40p | awk '{print $2}' | sed 's/.$//'`
CELLVOLTAGE6=`cat $OUTPUTFILE | sed -n 41p | awk '{print $2}' | sed 's/.$//'`
CELLVOLTAGE7=`cat $OUTPUTFILE | sed -n 42p | awk '{print $2}' | sed 's/.$//'`
CELLVOLTAGE8=`cat $OUTPUTFILE | sed -n 43p | awk '{print $2}'`
ERRORS=`cat $OUTPUTFILE | sed -n 52p`
if [ $ERRORS = "}" ]; then
	ERRORS="NoErrors"
fi

# Calculating the cell deviation between the cells, highest en lowest cell
CELLDEVIATION=`echo $HIGHESTVOLTAGE - $LOWESTVOLTAGE | bc`

# Display the main value's to console with clear value's
clear
echo "$INFO"
echo ""
echo "TOTALVOLTAGE            = $TOTALVOLTAGE V"
echo "TOTALCURRENT            = $TOTALCURRENT A"
echo "SOC                     = $SOC %"
echo "HIGHESTVOLTAGE          = $HIGHESTVOLTAGE V"
echo "HIGHESTCELL             = $HIGHESTCELL"
echo "LOWESTVOLTAGE           = $LOWESTVOLTAGE V"
echo "LOWESTCELL              = $LOWESTCELL"
echo "CELLDEVIATION           = $CELLDEVIATION V"
echo "TEMPERATURE             = $TEMPERATURE °C"
echo "CHARGINGMOSFET          = $CHARGINGMOSFET"
echo "DISCHARGINGMOSFET       = $DISCHARGINGMOSFET"
echo "CAPACITYAHLEFT          = $CAPACITYAHLEFT Ah"
echo "CYCLES                  = $CYCLES X"
echo "CELLVOLTAGE1            = $CELLVOLTAGE1 V"
echo "CELLVOLTAGE2            = $CELLVOLTAGE2 V"
echo "CELLVOLTAGE3            = $CELLVOLTAGE3 V"
echo "CELLVOLTAGE4            = $CELLVOLTAGE4 V"
echo "CELLVOLTAGE5            = $CELLVOLTAGE5 V"
echo "CELLVOLTAGE6            = $CELLVOLTAGE6 V"
echo "CELLVOLTAGE7            = $CELLVOLTAGE7 V"
echo "CELLVOLTAGE8            = $CELLVOLTAGE8 V"
echo "ERRORS                  = $ERRORS"

# Send the value's to Domoticz, if readings are not good skip it.
if [ -n "$TOTALVOLTAGE" ]; then
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$TOTALVOLTAGEIDX&nvalue=0&svalue=$TOTALVOLTAGE" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$TOTALCURRENTIDX&nvalue=0&svalue=$TOTALCURRENT" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$SOCIDX&nvalue=0&svalue=$SOC" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CAPACITYAHLEFTIDX&nvalue=0&svalue=$CAPACITYAHLEFT" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$TEMPERATUREIDX&nvalue=0&svalue=$TEMPERATURE" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CYCLESIDX&nvalue=0&svalue=$CYCLES" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CELLVOLTAGE1IDX&nvalue=0&svalue=$CELLVOLTAGE1" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CELLVOLTAGE2IDX&nvalue=0&svalue=$CELLVOLTAGE2" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CELLVOLTAGE3IDX&nvalue=0&svalue=$CELLVOLTAGE3" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CELLVOLTAGE4IDX&nvalue=0&svalue=$CELLVOLTAGE4" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CELLVOLTAGE5IDX&nvalue=0&svalue=$CELLVOLTAGE5" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CELLVOLTAGE6IDX&nvalue=0&svalue=$CELLVOLTAGE6" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CELLVOLTAGE7IDX&nvalue=0&svalue=$CELLVOLTAGE7" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CELLVOLTAGE8IDX&nvalue=0&svalue=$CELLVOLTAGE8" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CELLDEVIATIONIDX&nvalue=0&svalue=$CELLDEVIATION" >> /dev/null 2>&1


	# Get the domoticz value from the charging mosfet state switch if its on or off, and cut the result, otherwise you get multiple ON/OFF events in Domoticz
	# Also convert the value from true/false to on/off
	VALUEIDXINDOMOTICZCHARGINGMOSFET=`curl -s 'http://'$DOMOTICZIP':'$DOMOTICZPORT'/json.htm?type=devices&rid='$CHARGINGMOSFETIDX'' | /bin/grep Data | /usr/bin/awk '{ print $3 }' | /bin/sed 's/,/ /g' | /usr/bin/tr -d '"'`
	if  [ "$CHARGINGMOSFET" = "true" ]; then	

		VALUECHARGINGMOSFET=On
		# Check if value different otherwise you send the status every 1-5min
		if [ $VALUECHARGINGMOSFET != $VALUEIDXINDOMOTICZCHARGINGMOSFET ]; then
			curl -i -s 'http://'$DOMOTICZIP':'$DOMOTICZPORT'/json.htm?type=command&param=switchlight&idx='$CHARGINGMOSFETIDX'&switchcmd='$VALUECHARGINGMOSFET'' >> /dev/null 2>&1
		fi

		else
		VALUECHARGINGMOSFET=Off
		# Check if value different otherwise you send the status every 1-5min
		if [ $VALUECHARGINGMOSFET != $VALUEIDXINDOMOTICZCHARGINGMOSFET ]; then
			curl -i -s 'http://'$DOMOTICZIP':'$DOMOTICZPORT'/json.htm?type=command&param=switchlight&idx='$CHARGINGMOSFETIDX'&switchcmd='$VALUECHARGINGMOSFET'' >> /dev/null 2>&1
		fi
	fi

	# Get the domoticz value from the discharging mosfet state switch if its on or off, and cut the result, otherwise you get multiple ON/OFF events in Domoticz
	# Also convert the value from true/false to on/off
	VALUEIDXINDOMOTICZDISCHARGINGMOSFET=`curl -s 'http://'$DOMOTICZIP':'$DOMOTICZPORT'/json.htm?type=devices&rid='$DISCHARGINGMOSFETIDX'' | /bin/grep Data | /usr/bin/awk '{ print $3 }' | /bin/sed 's/,/ /g' | /usr/bin/tr -d '"'`
	if  [ "$DISCHARGINGMOSFET" = "true" ]; then	

		VALUEDISCHARGINGMOSFET=On
		# Check if value different otherwise you send the status every 1-5min
		if [ $VALUEDISCHARGINGMOSFET != $VALUEIDXINDOMOTICZDISCHARGINGMOSFET ]; then
			curl -i -s 'http://'$DOMOTICZIP':'$DOMOTICZPORT'/json.htm?type=command&param=switchlight&idx='$DISCHARGINGMOSFETIDX'&switchcmd='$VALUEDISCHARGINGMOSFET'' >> /dev/null 2>&1
		fi

		else
		VALUEDISCHARGINGMOSFET=Off
		# Check if value different otherwise you send the status every 1-5min
		if [ $VALUEDISCHARGINGMOSFET != $VALUEIDXINDOMOTICZDISCHARGINGMOSFET ]; then
			curl -i -s 'http://'$DOMOTICZIP':'$DOMOTICZPORT'/json.htm?type=command&param=switchlight&idx='$DISCHARGINGMOSFETIDX'&switchcmd='$VALUEDISCHARGINGMOSFET'' >> /dev/null 2>&1
		fi
	fi
fi

exit
#EOF
